﻿using Commons.Emulation;
using ICsvToBin;
using StockSharp.Algo.Storages;
using StockSharp.BusinessEntities;
using StockSharp.Remote;
using StorageTraders;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
//using QScalp;
using QScalp.History;
using QScalp.History.Reader;
using QScalp.History.Writer;
using Scalps;
using StockSharp.Messages;
using StorageDatas;
using HistoryCast;

namespace CsvToBinary
{
    //public class DomainFileGuid : MarshalByRefObject, IGuidFile
    public class CsvToBin : MarshalByRefObject, ICsvToBinary
    {
        private static Dictionary<string, string> _mapSecur = new Dictionary<string, string> 
        {
            {"GZ", "GAZR"},
            {"RI", "RTS"}
        };
        private IEnumerable<Security> GetSecures()
        {
            var nm = Path.Combine(MyFiles.ArcsFolder, "params");
            var secures = IOCandle.LoadSecurity(nm);
            return secures;
        }
        public string ConvertFromQsh(string securId, string folderQsh, string folderDest, DateTime start, 
            bool isSaveToBin, bool isVolume, bool isCalcVol, bool isFormCandle, Array frames)
        {
            var names = Directory.GetFiles(folderQsh, "*.qsh");
            var isForts = securId.Contains("FORTS");
            var sa = securId.Split(new char[] {'@'});
            var code = isForts ? sa[0].Substring(0,2) : sa[0];
            var fnma = names.Where(p => _mapSecur.ContainsKey(code) && p.Contains(_mapSecur[code])).ToArray();
            var dt = start;
            var dtstr = string.Format("{0,4:0000}-{1,2:00}-{2,2:00}", dt.Year, dt.Month, dt.Day);
            var fnm = fnma.FirstOrDefault(p => p.Contains(dtstr));
            if (fnm == null) return "File not found";
            var secures = StorageTraders.IOCandle.LoadSecurity(MyFiles.FolderParam);
            var secur_ = secures.FirstOrDefault(p => p.Id == securId);
            //var trades = secur_.GetTradesFromQuik(start);
            //var vol = trades.Where(p => p.Time.TimeOfDay <= new TimeSpan(10, 1, 0)).Sum(p => p.Volume);

            var save = new SaveMessages(fnm, folderDest, securId, dt)
            {
                Maps = _mapSecur,
                Secures = secures,
                IsSaveToBin = isSaveToBin,
                IsVolume = isVolume,
                Frames = frames,
                IsCalcVol = isCalcVol
                //ActSave = (s, m) => Save(s, m)
            };
            if (isFormCandle)
                save.ActSave = (s, m) => Save(s, m);
            save.Execute();
            return null;
        }
        private void Save(SaveMessages sv, IEnumerable<StockSharp.Messages.Message> msgs)
        {
            var msg = msgs.FirstOrDefault();
            if (msg != null && msg is ExecutionMessage)
            {
                  var  trades = msgs
                    .Select(p => new Trade()
                    {
                            Id = ((ExecutionMessage)p).TradeId != null ? ((ExecutionMessage)p).TradeId.Value : 0l,
                            Price = ((ExecutionMessage)p).TradePrice != null ? ((ExecutionMessage)p).TradePrice.Value : 0m,
                            Volume = ((ExecutionMessage)p).Volume != null ? ((ExecutionMessage)p).Volume.Value : 0m,
                            Security = sv.Secur,
                            Time = ((ExecutionMessage)p).LocalTime,
                            OrderDirection = ((ExecutionMessage)p).OriginSide != null ? ((ExecutionMessage)p).OriginSide.Value : (Sides?)null,
                            OpenInterest = ((ExecutionMessage)p).OpenInterest != null ? ((ExecutionMessage)p).OpenInterest.Value : (decimal?)null
                    }).ToArray();
                  Dictionary<FrameTick, Tuple<DateTime, DateTime>> dates = ((FrameTick[])sv.Frames)
                      .ToDictionary(p => p, p => new Tuple<DateTime, DateTime>(sv.Start.Date, sv.Start.Date));
                  var candles = sv.Secur.CreateCandleFromTrades(null, (FrameTick[])sv.Frames, dates, sv.IsCalcVol, false, sv.IsVolume, trades);
                  candles.SaveCandlesToDisk();
//public static List<Candle> CreateCandleFromTrades(this Security secur, string hist, 
//            FrameTick[] frames, Dictionary<FrameTick, Tuple<DateTime, DateTime>> dates,
//           bool isCalcProfile, bool isCsv, bool isVolume, IEnumerable<Trade> trades = null)
            }
            //if (msg != null)
            //{
            //    if (msg is ExecutionMessage) 
            //        HelperBinArc.SaveTradeMessage(msgs.Cast<ExecutionMessage>());
            //    else if (msg is QuoteChangeMessage)
            //        HelperBinArc.SaveQuotesMessage(msgs.Cast<QuoteChangeMessage>());
            //    else if (msg is Level1ChangeMessage)
            //        HelperBinArc.SaveLevel1Message(msgs.Cast<Level1ChangeMessage>());
            //}
        }
    }
}
